export class Main extends Indicator {

    constructor() {
        super();

        this.addInput("PeriodsATR", InputType.int, 18);
        this.addInput("MA_Periods", InputType.int, 49);
        this.addInput("MA_type", InputType.int, Averaging.LinearWeighted);
        this.addInput("Mult_Factor1", InputType.float, 1.6);
        this.addInput("Mult_Factor2", InputType.float, 3.2);
        this.addInput("Mult_Factor3", InputType.float, 4.8);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffers = {
            MA_Buffer0 : this.addBuffer(),
            Ch1up_Buffer1 : this.addBuffer(),
            Ch1dn_Buffer2 : this.addBuffer(),
            Ch2up_Buffer3 : this.addBuffer(),
            Ch2dn_Buffer4 : this.addBuffer(),
            Ch3up_Buffer5 : this.addBuffer(),
            Ch3dn_Buffer6 : this.addBuffer(),
        };
    }

    onInit() {
        let {PeriodsATR, MA_Periods, Mult_Factor1, Mult_Factor2, Mult_Factor3} = this.getInputs(),
            drawBegin = Math.max(PeriodsATR, MA_Periods);

        this.buffers.MA_Buffer0
            .setShape(Shape.Line)
            .setColor(Color.Green)
            .setDrawBegin(drawBegin)
        ;

        this.buffers.Ch1up_Buffer1
            .setShape(Shape.Line)
            .setColor(Color.DeepSkyBlue)
            .setDrawBegin(drawBegin)
            .setLabel("ATRd("+PeriodsATR+", "+Mult_Factor1+")")
        ;

        this.buffers.Ch1dn_Buffer2
            .setShape(Shape.Line)
            .setColor(Color.DeepSkyBlue)
            .setDrawBegin(drawBegin)
            .setLabel("ATRd("+PeriodsATR+", "+Mult_Factor1+")")
        ;

        this.buffers.Ch2up_Buffer3
            .setShape(Shape.Line)
            .setColor(Color.Blue)
            .setDrawBegin(drawBegin)
            .setLabel("ATRd("+PeriodsATR+", "+Mult_Factor2+")")
        ;

        this.buffers.Ch2dn_Buffer4
            .setShape(Shape.Line)
            .setColor(Color.Blue)
            .setDrawBegin(drawBegin)
            .setLabel("ATRd("+PeriodsATR+", "+Mult_Factor2+")")
        ;

        this.buffers.Ch3up_Buffer5
            .setShape(Shape.Line)
            .setColor(Color.Red)
            .setDrawBegin(drawBegin)
            .setLabel("ATRd("+PeriodsATR+", "+Mult_Factor3+")")
        ;

        this.buffers.Ch3dn_Buffer6
            .setShape(Shape.Line)
            .setColor(Color.Red)
            .setDrawBegin(drawBegin)
            .setLabel("ATRd("+PeriodsATR+", "+Mult_Factor3+")")
        ;
    }

    async onUpdate() {
        const {PeriodsATR, MA_Periods, MA_type, Mult_Factor1, Mult_Factor2, Mult_Factor3, PriceType} = this.getInputs(),
              {MA_Buffer0, Ch1up_Buffer1, Ch1dn_Buffer2, Ch2up_Buffer3, Ch2dn_Buffer4, Ch3up_Buffer5, Ch3dn_Buffer6} = this.buffers,
            {
                High,
                Low,
                Close,
            } = await Bar.load([
                Bar.Mode.High,
                Bar.Mode.Low,
                Bar.Mode.Close,
            ], PriceType),
            barSize = Close.length;

        let maList, atrList, atr, ma;

        if (barSize <= Math.max(PeriodsATR, MA_Periods)) {
            return;
        }

        maList = await Indicators.MA(Current.Symbol, Current.TimeFrame, MA_Periods, 0, MA_type, AppliedPrice.Typical, PriceType);
        atrList = await Indicators.ATR(Current.Symbol, Current.TimeFrame, PeriodsATR, PriceType);

        for (let i = 0; i < barSize; i++) {
            atr = atrList[i];
            ma = maList[i];
            MA_Buffer0.set(i, ma);
            Ch1up_Buffer1.set(i, (ma + atr * Mult_Factor1));
            Ch1dn_Buffer2.set(i, (ma - atr * Mult_Factor1));
            Ch2up_Buffer3.set(i, (ma + atr * Mult_Factor2));
            Ch2dn_Buffer4.set(i, (ma - atr * Mult_Factor2));
            Ch3up_Buffer5.set(i, (ma + atr * Mult_Factor3));
            Ch3dn_Buffer6.set(i, (ma - atr * Mult_Factor3));
        }
    }
}